using System.Reflection;
using System.Runtime.CompilerServices;
using System.Diagnostics;
using System.Data;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Security.Permissions;
using Microsoft.Win32;
using System;
using System.ComponentModel;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;
using Microsoft.VisualBasic;
using Microsoft.InteropFormTools;


namespace IOPCodeCamp
{
#if COM_INTEROP_ENABLED
	
	//Adds the InteropToolbox to the My namespace
	namespace My
	{
		//The HideModuleNameAttribute hides the module name MyInteropToolbox so the syntax becomes My.InteropToolbox.
		[global::Microsoft.VisualBasic.HideModuleName()]sealed class MyInteropToolbox
		{
			
			private static InteropToolbox _toolbox = new InteropToolbox();
			
			public static InteropToolbox InteropToolbox
			{
				get
				{
					return _toolbox;
				}
			}
		}
	}
	
	//Helper routines to do additional registration needed by ActiveX controls.
	internal sealed class ComRegistration
	{
		
		const int OLEMISC_RECOMPOSEONRESIZE = 1;
		const int OLEMISC_CANTLINKINSIDE = 16;
		const int OLEMISC_INSIDEOUT = 128;
		const int OLEMISC_ACTIVATEWHENVISIBLE = 256;
		const int OLEMISC_SETCLIENTSITEFIRST = 131072;
		
		public static void RegisterControl(Type t)
		{
			
			try
			{
				GuardNullType(t, "t");
				GuardTypeIsControl(t);
				
				//CLSID
				string key = (string) ("CLSID\\" + t.GUID.ToString("B"));
				
				using (RegistryKey subkey = Registry.ClassesRoot.OpenSubKey(key, true))
				{
					
					//InProcServer32
					RegistryKey InprocKey = subkey.OpenSubKey("InprocServer32", true);
					if (InprocKey != null)
					{
						InprocKey.SetValue(null, Environment.SystemDirectory + "\\mscoree.dll");
					}
					
					//Control
					using (RegistryKey controlKey = subkey.CreateSubKey("Control"))
					{
					}
					
					
					//Misc
					using (RegistryKey miscKey = subkey.CreateSubKey("MiscStatus"))
					{
						int MiscStatusValue = System.Convert.ToInt32(OLEMISC_RECOMPOSEONRESIZE + OLEMISC_CANTLINKINSIDE + OLEMISC_INSIDEOUT + OLEMISC_ACTIVATEWHENVISIBLE + OLEMISC_SETCLIENTSITEFIRST);
						
						miscKey.SetValue("", MiscStatusValue.ToString(), RegistryValueKind.String);
					}
					
					
					//ToolBoxBitmap32
					using (RegistryKey bitmapKey = subkey.CreateSubKey("ToolBoxBitmap32"))
					{
						
						//If you want to have different icons for each control in this assembly
						//you can modify this section to specify a different icon each time.
						//Each specified icon must be embedded as a win32resource in the
						//assembly; the default one is at index 101, but you can additional ones.
						bitmapKey.SetValue("", Assembly.GetExecutingAssembly().Location + ", 101", RegistryValueKind.String);
					}
					
					
					//TypeLib
					using (RegistryKey typeLibKey = subkey.CreateSubKey("TypeLib"))
					{
						Guid libId = Marshal.GetTypeLibGuidForAssembly(t.Assembly);
						typeLibKey.SetValue("", libId.ToString("B"), RegistryValueKind.String);
					}
					
					
					//Version
					using (RegistryKey versionKey = subkey.CreateSubKey("Version"))
					{
						int major;
						int minor;
						Marshal.GetTypeLibVersionForAssembly(t.Assembly, out major, out minor);
						versionKey.SetValue("", string.Format("{0}.{1}", major, minor));
					}
					
					
				}
				
				
			}
			catch (Exception ex)
			{
				LogAndRethrowException("ComRegisterFunction failed.", t, ex);
			}
			
		}
		
		public static void UnregisterControl(Type t)
		{
			try
			{
				GuardNullType(t, "t");
				GuardTypeIsControl(t);
				
				//CLSID
				string key = (string) ("CLSID\\" + t.GUID.ToString("B"));
				Registry.ClassesRoot.DeleteSubKeyTree(key);
				
			}
			catch (Exception ex)
			{
				LogAndRethrowException("ComUnregisterFunction failed.", t, ex);
			}
			
		}
		
		private static void GuardNullType(Type t, string param)
		{
			if (t == null)
			{
				throw (new ArgumentException("The CLR type must be specified.", param));
			}
		}
		
		private static void GuardTypeIsControl(Type t)
		{
			if (! typeof(Control).IsAssignableFrom(t))
			{
				throw (new ArgumentException("Type argument must be a Windows Forms control."));
			}
		}
		
		private static void LogAndRethrowException(string message, Type t, Exception ex)
		{
			try
			{
				if (t != null)
				{
					message += (string) ("\r\n" + string.Format("CLR class \'{0}\'", t.FullName));
				}
				
				throw (new ComRegistrationException(message, ex));
				
			}
			catch (Exception ex2)
			{
				(new Microsoft.VisualBasic.ApplicationServices.ConsoleApplicationBase()).Log.WriteException(ex2);
			}
			
		}
		
	}
	
	[Serializable()]public class ComRegistrationException : Exception
	{
		
		public ComRegistrationException()
		{
			
		}
		
		public ComRegistrationException(string message, Exception inner) : base(message, inner)
		{
		}
		
	}
	
	//Helper functions to convert common COM types to their .NET equivalents
	[ComVisible(false)]internal class ActiveXControlHelpers : System.Windows.Forms.AxHost
	{
		
		internal ActiveXControlHelpers() : base(null)
		{
		}
		
		internal static Color GetColorFromOleColor(int oleColor)
		{
			return AxHost.GetColorFromOleColor(CIntToUInt(oleColor));
		}
		
		internal static new int GetOleColorFromColor(Color color)
		{
			return CUIntToInt(AxHost.GetOleColorFromColor(color));
		}
		
		internal static int CUIntToInt(UInt32 uiArg)
		{
			if (uiArg <= int.MaxValue)
			{
				return System.Convert.ToInt32(uiArg);
			}
			return System.Convert.ToInt32(uiArg - 2 * (((uint) int.MaxValue) + 1));
		}
		
		internal static UInt32 CIntToUInt(int iArg)
		{
			if (iArg < 0)
			{
				return System.Convert.ToUInt32(UInt32.MaxValue + iArg + 1);
			}
			return (uint) iArg;
		}
		
		private const int KEY_PRESSED = 0x1000;
		[DllImport("user32",EntryPoint="GetKeyState", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		private static extern short GetKeyState(int ByValnVirtKey);
		
		private static int CheckForAccessorKey()
		{
			if ((new Microsoft.VisualBasic.Devices.Computer()).Keyboard.AltKeyDown)
			{
				
				for (int i =  (int) (Keys.A); i <= System.Convert.ToInt32(Keys.Z); i++)
				{
					if ((GetKeyState(i) & KEY_PRESSED) != 0)
					{
						return i;
					}
				}
			}
			
			return -1;
		}
		
		[ComVisible(false)]internal static void HandleFocus(UserControl f)
		{
			if ((new Microsoft.VisualBasic.Devices.Computer()).Keyboard.AltKeyDown)
			{
				HandleAccessorKey(f.GetNextControl(null, true), f);
			}
			else
			{
				//Move to the first control that can receive focus, taking into account
				//the possibility that the user pressed <Shift>+<Tab>, in which case we
				//need to start at the end and work backwards.
				Control ctl = f.GetNextControl(null, System.Convert.ToBoolean(! (new Microsoft.VisualBasic.Devices.Computer()).Keyboard.ShiftKeyDown));
				while (ctl != null)
				{
					if (ctl.Enabled && ctl.CanSelect)
					{
						ctl.Focus();
						break;
					}
					else
					{
						ctl = f.GetNextControl(ctl, System.Convert.ToBoolean(! (new Microsoft.VisualBasic.Devices.Computer()).Keyboard.ShiftKeyDown));
					}
				}
				
			}
		}
		
		private static void HandleAccessorKey(object sender, UserControl f)
		{
			int key = CheckForAccessorKey();
			if (key == -1)
			{
				return;
			}
			
			Control ctlCurrent = f.GetNextControl((Control) sender, false);
			
			do
			{
				ctlCurrent = f.GetNextControl(ctlCurrent, true);
				if (ctlCurrent != null && Control.IsMnemonic(Strings.ChrW(key), ctlCurrent.Text))
				{
					
					//VB6 handles conflicts correctly already, so if we handle it also we'll end up
					//one control past where the focus should be
					if (! KeyConflict(Strings.ChrW(key), f))
					{
						
						//If we land on a label or other non-selectable control then go to the next
						//control in the tab order
						if (! ctlCurrent.CanSelect)
						{
							Control ctlAfterLabel = f.GetNextControl(ctlCurrent, true);
							if (ctlAfterLabel != null && ctlAfterLabel.CanFocus)
							{
								ctlAfterLabel.Focus();
							}
						}
						else
						{
							ctlCurrent.Focus();
						}
						break;
					}
				}
				
				//Loop until we hit the end of the tab order
				//If we've hit the end of the tab order we don't want to loop back because the
				//parent form's controls come next in the tab order.
			} while (!(ctlCurrent == null));
		}
		
		private static bool KeyConflict(char key, UserControl u)
		{
			bool flag = false;
			
			foreach (Control ctl in u.Controls)
			{
				if (Control.IsMnemonic(key, ctl.Text))
				{
					if (flag)
					{
						return true;
					}
					flag = true;
				}
			}
			return false;
		}
		
		//Handles <Tab> and <Shift>+<Tab>
		internal static void TabHandler(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Tab)
			{
				Control ctl = (Control) sender;
				
				UserControl userCtl = GetParentUserControl(ctl);
				
				Control firstCtl = userCtl.GetNextControl(null, true);
				while (!(firstCtl == null || firstCtl.CanSelect))
				{
					firstCtl = userCtl.GetNextControl(firstCtl, true);
				}
				
				Control lastCtl = userCtl.GetNextControl(null, false);
				while (!(lastCtl == null || lastCtl.CanSelect))
				{
					lastCtl = userCtl.GetNextControl(lastCtl, false);
				}
				
				if (ctl == lastCtl || ctl == firstCtl || lastCtl.Contains(ctl) || firstCtl.Contains(ctl))
				{
					
					userCtl.SelectNextControl((Control) sender, lastCtl == userCtl.ActiveControl, true, true, true);
				}
			}
		}
		
		private static UserControl GetParentUserControl(Control ctl)
		{
			if (ctl == null)
			{
				return null;
			}
			
			while (!(ctl.Parent == null))
			{
				ctl = ctl.Parent;
			}
			if (ctl != null)
			{
				return ((UserControl) ctl);
			}
			
			return null;
		}
		
		internal static void WireUpHandlers(Control ctl, EventHandler ValidationHandler)
		{
			if (ctl != null)
			{
				ctl.KeyDown += new System.Windows.Forms.KeyEventHandler(ActiveXControlHelpers.TabHandler);
				ctl.LostFocus += new System.EventHandler(ValidationHandler);
				
				if (ctl.HasChildren)
				{
					foreach (Control child in ctl.Controls)
					{
						WireUpHandlers(child, ValidationHandler);
					}
				}
			}
		}
		
	}
	
#endif
}
