using System.Reflection;
using System.Runtime.CompilerServices;
using System.Diagnostics;
using System.Data;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Security.Permissions;
using Microsoft.Win32;
using System;
using System.ComponentModel;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;
using Microsoft.VisualBasic;


namespace IOPCodeCamp
{
	[ComSourceInterfaces(typeof(IInteropUserControlEvents))]public partial class IOPControl
	{
		
		//<ComClass(InteropUserControl.ClassId, InteropUserControl.InterfaceId, InteropUserControl.EventsId)> _
#region VB6 Interop Code
		
#if COM_INTEROP_ENABLED
		
#region COM Registration
		
		// These  GUIDs provide the COM identity for this class
		// and its COM interfaces. If you change them, existing
		// clients will no longer be able to access the class.
		
		public const string ClassId = "a80154bb-af93-4bab-a8da-fa061d2a7d7d";
		public const string InterfaceId = "77e5d80e-fc76-46ae-a3b9-23022a3db55e";
		public const string EventsId = "b9004b78-a2ba-4792-ba51-84f42d89b6b9";
		
		//These routines perform the additional COM registration needed by ActiveX controls
		[EditorBrowsable(EditorBrowsableState.Never)][ComRegisterFunction()]private static void Register(Type t)
		{
			ComRegistration.RegisterControl(t);
		}
		
		[EditorBrowsable(EditorBrowsableState.Never)][ComUnregisterFunction()]private static void Unregister(Type t)
		{
			ComRegistration.UnregisterControl(t);
		}
		
#endregion
		
#region VB6 Events
		
		//This section shows some examples of exposing a UserControl's events to VB6.  Typically, you just
		//1) Declare the event as you want it to be shown in VB6
		//2) Raise the event in the appropriate UserControl event.
		
		public delegate void ClickEventHandler(); //Event must be marked as Shadows since .NET UserControls have the same name.
		private ClickEventHandler ClickEvent;
		
		public new event ClickEventHandler Click
		{
			add
			{
				ClickEvent = (ClickEventHandler) System.Delegate.Combine(ClickEvent, value);
			}
			remove
			{
				ClickEvent = (ClickEventHandler) System.Delegate.Remove(ClickEvent, value);
			}
		}
		
		public delegate void DblClickEventHandler();
		private DblClickEventHandler DblClickEvent;
		
		public event DblClickEventHandler DblClick
		{
			add
			{
				DblClickEvent = (DblClickEventHandler) System.Delegate.Combine(DblClickEvent, value);
			}
			remove
			{
				DblClickEvent = (DblClickEventHandler) System.Delegate.Remove(DblClickEvent, value);
			}
		}
		
		
		public void InteropUserControl_Click(object sender, System.EventArgs e)
		{
			if (ClickEvent != null)
				ClickEvent();
		}
		
		public void InteropUserControl_DoubleClick(object sender, System.EventArgs e)
		{
			if (DblClickEvent != null)
				DblClickEvent();
		}
		
#endregion
		
#region VB6 Properties
		
		//The following are examples of how to expose typical form properties to VB6.
		//You can also use these as examples on how to add additional properties.
		
		//Must Shadow this property as it exists in Windows.Forms and is not overridable
		public new bool Visible
		{
			get
			{
				return base.Visible;
			}
			set
			{
				base.Visible = value;
			}
		}
		
		public new bool Enabled
		{
			get
			{
				return base.Enabled;
			}
			set
			{
				base.Enabled = value;
			}
		}
		
		public int ForegroundColor
		{
			get
			{
				return ActiveXControlHelpers.GetOleColorFromColor(base.ForeColor);
			}
			set
			{
				base.ForeColor = ActiveXControlHelpers.GetColorFromOleColor(value);
			}
		}
		
		public int BackgroundColor
		{
			get
			{
				return ActiveXControlHelpers.GetOleColorFromColor(base.BackColor);
			}
			set
			{
				base.BackColor = ActiveXControlHelpers.GetColorFromOleColor(value);
			}
		}
		
		public override System.Drawing.Image BackgroundImage
		{
			get
			{
				return null;
			}
			set
			{
				if (value != null)
				{
					Interaction.MsgBox("Setting the background image of an Interop UserControl is not supported, please use a PictureBox instead.", MsgBoxStyle.Information, null);
				}
				base.BackgroundImage = null;
			}
		}
		
#endregion
		
#region VB6 Methods
		
		public override void Refresh()
		{
			base.Refresh();
		}
		
		//Ensures that tabbing across VB6 and .NET controls works as expected
		public void UserControl_LostFocus(object sender, System.EventArgs e)
		{
			ActiveXControlHelpers.HandleFocus(this);
		}
		
		public IOPControl()
		{
			
			// This call is required by the Windows Form Designer.
			InitializeComponent();
			
			// Add any initialization after the InitializeComponent() call.
			
			//Raise Load event
			this.OnCreateControl();
		}
		
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]protected override void WndProc(ref System.Windows.Forms.Message m)
		{
			
			const int WM_SETFOCUS = 0x7;
			const int WM_PARENTNOTIFY = 0x210;
			const int WM_DESTROY = 0x2;
			const int WM_LBUTTONDOWN = 0x201;
			const int WM_RBUTTONDOWN = 0x204;
			
			if (m.Msg == WM_SETFOCUS)
			{
				//Raise Enter event
				this.OnEnter(new System.EventArgs());
				
			}
			else if (m.Msg == WM_PARENTNOTIFY && (m.WParam.ToInt32() == WM_LBUTTONDOWN || m.WParam.ToInt32() == WM_RBUTTONDOWN))
			{
				
				if (! this.ContainsFocus)
				{
					//Raise Enter event
					this.OnEnter(new System.EventArgs());
				}
				
			}
			else if (m.Msg == WM_DESTROY && ! this.IsDisposed && ! this.Disposing)
			{
				//Used to ensure that VB6 will cleanup control properly
				this.Dispose();
			}
			
			base.WndProc(ref m);
		}
		
		//This event will hook up the necessary handlers
		public void InteropUserControl_ControlAdded(object sender, ControlEventArgs e)
		{
			ActiveXControlHelpers.WireUpHandlers(e.Control, new EventHandler(ValidationHandler));
		}
		
		//Ensures that the Validating and Validated events fire appropriately
		internal void ValidationHandler(object sender, EventArgs e)
		{
			
			if (this.ContainsFocus)
			{
				return;
			}
			
			//Raise Leave event
			this.OnLeave(e);
			
			if (this.CausesValidation)
			{
				CancelEventArgs validationArgs = new CancelEventArgs();
				this.OnValidating(validationArgs);
				
				if (validationArgs.Cancel && this.ActiveControl != null)
				{
					this.ActiveControl.Focus();
				}
				else
				{
					//Raise Validated event
					this.OnValidated(e);
				}
			}
			
		}
		
#endregion
		
#endif
		
#endregion
		
		//Please enter any new code here, below the Interop code
		
		// event to show in PB
		public delegate void tbScrollEventHandler();
		private tbScrollEventHandler tbScrollEvent;
		
		public event tbScrollEventHandler tbScroll
		{
			add
			{
				tbScrollEvent = (tbScrollEventHandler) System.Delegate.Combine(tbScrollEvent, value);
			}
			remove
			{
				tbScrollEvent = (tbScrollEventHandler) System.Delegate.Remove(tbScrollEvent, value);
			}
		}
		
		public delegate void tbValuechangedEventHandler();
		private tbValuechangedEventHandler tbValuechangedEvent;
		
		public event tbValuechangedEventHandler tbValuechanged
		{
			add
			{
				tbValuechangedEvent = (tbValuechangedEventHandler) System.Delegate.Combine(tbValuechangedEvent, value);
			}
			remove
			{
				tbValuechangedEvent = (tbValuechangedEventHandler) System.Delegate.Remove(tbValuechangedEvent, value);
			}
		}
		
		
		public void TB1_Scroll(System.Object sender, System.EventArgs e)
		{
			Label1.Text = TB1.Value.ToString();
			if (tbScrollEvent != null)
				tbScrollEvent();
		}
		public void TB1_Valuechanged(System.Object sender, System.EventArgs e)
		{
			if (tbValuechangedEvent != null)
				tbValuechangedEvent();
		}
		
		
		// method to call from PB
		public void tbSetBackColor(int testval)
		{
			Color myColor = ColorTranslator.FromWin32(testval);
			
			TB1.BackColor = myColor;
			Label1.BackColor = myColor;
			BackColor = myColor;
		}
		// Property get/set
		public int TB1BackgroundColor
		{
			get
			{
				return ActiveXControlHelpers.GetOleColorFromColor(TB1.BackColor);
			}
			set
			{
				TB1.BackColor = ActiveXControlHelpers.GetColorFromOleColor(value);
			}
		}
		public int TB1Value
		{
			get
			{
				return TB1.Value;
			}
			set
			{
				TB1.Value = value;
			}
		}
		
	}
}
